import 'package:flutter/material.dart';

import 'package:forme/forme.dart';

typedef FormeDateRangeTriggerBuilder = Widget Function(
    FormeDateRangeFieldState state);

/// example
///
/// ``` dart
///  FormeDateRangeField(
///     validator:
///          FormeValidates.notNull(errorText: 'range date required'),
///      triggerBuilder: (state) {
///        return InkWell(
///          focusNode: state.focusNode,
///          onTap: state.showPicker,
///         child: InputDecorator(
///           isFocused: state.hasFocus,
///           isEmpty: state.value == null,
///           decoration: InputDecoration(
///               errorText: state.errorText,
///               labelText: 'DateRange Picker'),
///           child: Text(state.value?.toString() ?? ''),
///         ),
///       );
///     },
///     name: 'dateRange',
///  )
///
class FormeDateRangeField extends FormeField<DateTimeRange?> {
  FormeDateRangeField({
    DateTimeRange? initialValue,
    String? name,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    bool enabled = true,
    FormeFieldDecorator<DateTimeRange?>? decorator,
    FormeFieldValidationFilter<DateTimeRange?>? validationFilter,
    FocusNode? focusNode,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<DateTimeRange?>? onStatusChanged,
    FormeFieldInitialized<DateTimeRange?>? onInitialized,
    FormeFieldSetter<DateTimeRange?>? onSaved,
    FormeValidator<DateTimeRange?>? validator,
    FormeAsyncValidator<DateTimeRange?>? asyncValidator,
    bool requestFocusOnUserInteraction = true,
    this.cancelText,
    this.confirmText,
    this.errorFormatText,
    this.errorInvalidText,
    this.helpText,
    this.initialEntryMode,
    this.routeSettings,
    this.use24hFormat = false,
    this.textDirection,
    this.locale,
    this.useRootNavigator = true,
    this.anchorPoint,
    this.errorInvalidRangeText,
    this.fieldEndHintText,
    this.fieldEndLabelText,
    this.fieldStartHintText,
    this.fieldStartLabelText,
    this.saveText,
    this.transitionBuilder,
    this.firstDate,
    this.lastDate,
    required this.triggerBuilder,
  }) : super.allFields(
          comparator: null,
          requestFocusOnUserInteraction: requestFocusOnUserInteraction,
          focusNode: focusNode,
          validationFilter: validationFilter,
          enabled: enabled,
          decorator: decorator,
          order: order,
          quietlyValidate: quietlyValidate,
          asyncValidatorDebounce: asyncValidatorDebounce,
          autovalidateMode: autovalidateMode,
          onStatusChanged: onStatusChanged,
          onInitialized: onInitialized,
          onSaved: onSaved,
          validator: validator,
          asyncValidator: asyncValidator,
          key: key,
          name: name,
          readOnly: readOnly,
          initialValue: initialValue,
          builder: (baseState) {
            return triggerBuilder.call(baseState as FormeDateRangeFieldState);
          },
        );

  final DateTime? firstDate;
  final DateTime? lastDate;
  final FormeDateRangeTriggerBuilder triggerBuilder;
  final String? helpText;
  final String? cancelText;
  final String? confirmText;
  final RouteSettings? routeSettings;
  final String? errorFormatText;
  final String? errorInvalidText;
  final DatePickerEntryMode? initialEntryMode;
  final TransitionBuilder? transitionBuilder;
  final bool use24hFormat;
  final TextDirection? textDirection;
  final Locale? locale;
  final bool useRootNavigator;
  final Offset? anchorPoint;
  final String? errorInvalidRangeText;
  final String? saveText;
  final String? fieldStartHintText;
  final String? fieldEndHintText;
  final String? fieldStartLabelText;
  final String? fieldEndLabelText;

  @override
  FormeDateRangeFieldState createState() => FormeDateRangeFieldState();
}

class FormeDateRangeFieldState extends FormeFieldState<DateTimeRange?> {
  @override
  FormeDateRangeField get widget => super.widget as FormeDateRangeField;

  VoidCallback? get showPicker => readOnly ? null : _pickRange;

  @override
  DateTimeRange? get value {
    final DateTimeRange? value = super.value;
    if (value == null) {
      return null;
    }
    return DateTimeRange(start: _simple(value.start), end: _simple(value.end));
  }

  void _pickRange() {
    showDateRangePicker(
      useRootNavigator: widget.useRootNavigator,
      anchorPoint: widget.anchorPoint,
      locale: widget.locale,
      initialDateRange: value,
      context: context,
      firstDate: widget.firstDate ?? DateTime(1970),
      lastDate: widget.lastDate ?? DateTime(2099),
      builder: widget.transitionBuilder,
      initialEntryMode: widget.initialEntryMode ?? DatePickerEntryMode.calendar,
      helpText: widget.helpText,
      cancelText: widget.cancelText,
      confirmText: widget.confirmText,
      saveText: widget.saveText,
      errorFormatText: widget.errorFormatText,
      errorInvalidText: widget.errorInvalidText,
      errorInvalidRangeText: widget.errorInvalidRangeText,
      fieldStartHintText: widget.fieldStartHintText,
      fieldEndHintText: widget.fieldEndHintText,
      fieldStartLabelText: widget.fieldStartLabelText,
      fieldEndLabelText: widget.fieldEndLabelText,
      routeSettings: widget.routeSettings,
      textDirection: widget.textDirection,
    ).then((value) {
      if (value != null) {
        didChange(value);
      }
      requestFocusOnUserInteraction();
    });
  }

  DateTime _simple(DateTime dateTime) {
    return DateTime(dateTime.year, dateTime.month, dateTime.day);
  }
}
